"use strict";

function _ramda() {
  var data = _interopRequireDefault(require("ramda"));

  _ramda = function _ramda() {
    return data;
  };

  return data;
}

function _utils() {
  var data = require("../../utils");

  _utils = function _utils() {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

/**
 * this file had been forked from https://github.com/dependents/node-dependency-tree
 */
var precinct = require('../precinct');

var fs = require('fs');

var cabinet = require('../filing-cabinet');

var debug = require('debug')('tree');

var Config = require('./Config');
/**
 * Recursively find all dependencies (avoiding circular) traversing the entire dependency tree
 * and returns a flat list of all unique, visited nodes
 *
 * @param {Object} options
 * @param {String} options.filename - The path of the module whose tree to traverse
 * @param {String} options.directory - The directory containing all JS files
 * @param {String} [options.requireConfig] - The path to a requirejs config
 * @param {String} [options.webpackConfig] - The path to a webpack config
 * @param {Object} [options.visited] - Cache of visited, absolutely pathed files that should not be reprocessed.
 *                             Format is a filename -> tree as list lookup table
 * @param {Array} [options.nonExistent] - List of partials that do not exist
 * @return {Object}
 */


module.exports = function (options) {
  var config = new Config(options);

  if (!fs.existsSync(config.filename)) {
    debug(`file ${config.filename} does not exist`);
    return {};
  }

  return traverse(config);
};
/**
 * Executes a post-order depth first search on the dependency tree and returns a
 * list of absolute file paths. The order of files in the list will be the
 * proper concatenation order for bundling.
 *
 * In other words, for any file in the list, all of that file's dependencies (direct or indirect) will appear at
 * lower indices in the list. The root (entry point) file will therefore appear last.
 *
 * The list will not contain duplicates.
 *
 * Params are those of module.exports
 */


module.exports.toList = function (options) {
  options.isListForm = true;
  return module.exports(options);
};
/**
 * Returns the list of dependencies for the given filename
 *
 * Protected for testing
 *
 * @param  {Config} config
 * @return {Array}
 */


module.exports._getDependencies = function (config) {
  var dependenciesRaw; // from some detectives it comes as an array, from some it is an object

  var precinctOptions = config.detectiveConfig;
  precinctOptions.includeCore = false;
  delete precinct.ast;

  try {
    dependenciesRaw = precinct.paperwork(config.filename, precinctOptions);
  } catch (e) {
    debug(`error getting dependencies: ${e.message}`);
    debug(e.stack);
    e.code = 'PARSING_ERROR';
    config.errors[config.filename] = e;
    dependenciesRaw = [];
  }

  var dependencies = _ramda().default.is(Object, dependenciesRaw) && !Array.isArray(dependenciesRaw) ? Object.keys(dependenciesRaw) : dependenciesRaw;
  var isDependenciesArray = Array.isArray(dependenciesRaw);
  debug(`extracted ${dependencies.length} dependencies: `, dependencies);
  var resolvedDependencies = [];
  var pathMapDependencies = [];
  var pathMapFile = {
    file: config.filename
  };
  dependencies.forEach(function (dependency) {
    return processDependency(dependency);
  });
  pathMapFile.dependencies = pathMapDependencies;
  config.pathMap.push(pathMapFile);
  return resolvedDependencies;

  function processDependency(dependency) {
    var cabinetParams = {
      partial: dependency,
      filename: config.filename,
      directory: config.directory,
      ast: precinct.ast,
      config: config.requireConfig,
      webpackConfig: config.webpackConfig,
      resolveConfig: config.resolveConfig
    };

    if (!isDependenciesArray && dependenciesRaw[dependency].isScript !== undefined) {
      // used for vue
      cabinetParams.isScript = dependenciesRaw[dependency].isScript;
    }

    var result;

    try {
      result = cabinet(cabinetParams);
    } catch (err) {
      debug(`error resolving dependencies: ${err.message}`);
      debug(err.stack);
      err.code = 'RESOLVE_ERROR';
      throw err;
    }

    if (!result) {
      debug(`skipping an empty filepath resolution for partial: ${dependency}`);
      addToNonExistent(dependency);
      return;
    }

    var exists = fs.existsSync(result);

    if (!exists) {
      addToNonExistent(dependency);
      debug(`skipping non-empty but non-existent resolution: ${result} for partial: ${dependency}`);
      return;
    }

    var pathMap = {
      importSource: dependency,
      resolvedDep: result
    };

    if (!isDependenciesArray && dependenciesRaw[dependency].importSpecifiers) {
      pathMap.importSpecifiers = dependenciesRaw[dependency].importSpecifiers;
    }

    if (!(0, _utils().isRelativeImport)(dependency) && config.resolveConfig) {
      // is includes also packages, which actually don't use customResolve, however, they will be
      // filtered out later.
      pathMap.isCustomResolveUsed = true;
    }

    pathMapDependencies.push(pathMap);
    resolvedDependencies.push(result);
  }

  function addToNonExistent(dependency) {
    if (config.nonExistent[config.filename]) {
      config.nonExistent[config.filename].push(dependency);
    } else {
      config.nonExistent[config.filename] = [dependency];
    }
  }
};
/**
 * the traverse is not a recursive function anymore, it has been changed to be iterative to fix
 * some performance issues.
 * @todo: we have some redundancy here with the `tree` and pathMap`. the `tree` has only the dependencies,
 * `pathMap` has the dependencies and some more info, such as importSpecifiers. we should use only
 * pathMap and get rid of tree.
 */


function traverse(config) {
  var tree = [];
  var stack = [config.filename];

  while (stack.length) {
    var dependency = stack.pop();
    debug(`traversing ${dependency}`);

    if (config.visited[dependency]) {
      populateFromCache(dependency);
    } else {
      traverseDependency(dependency);
    }
  }

  return tree;

  function traverseDependency(dependency) {
    var localConfig = config.clone();
    localConfig.filename = dependency;

    var dependencies = module.exports._getDependencies(localConfig);

    if (config.filter) {
      debug('using filter function to filter out dependencies');
      debug(`number of dependencies before filtering: ${dependencies.length}`);
      dependencies = dependencies.filter(function (filePath) {
        return localConfig.filter(filePath, localConfig.filename);
      });
      debug(`number of dependencies after filtering: ${dependencies.length}`);
    }

    debug('cabinet-resolved all dependencies: ', dependencies);
    tree[dependency] = dependencies;
    var filePathMap = config.pathMap.find(function (pathMapEntry) {
      return pathMapEntry.file === dependency;
    });
    if (!filePathMap) throw new Error(`file ${dependency} is missing from PathMap`);
    config.visited[dependency] = {
      pathMap: filePathMap,
      missing: config.nonExistent[dependency],
      error: config.errors[dependency]
    };
    stack.push.apply(stack, _toConsumableArray(dependencies));
  }

  function populateFromCache(dependency) {
    debug(`already visited ${dependency}. Will try to find it and its dependencies in the cache`);
    var dependenciesStack = [dependency];

    while (dependenciesStack.length) {
      findAllDependenciesInCache(dependenciesStack);
    }
  }

  function findAllDependenciesInCache(dependenciesStack) {
    var dependency = dependenciesStack.pop();

    if (!config.visited[dependency]) {
      debug(`unable to find ${dependency} in the cache, it was probably filtered before`);
      return;
    }

    debug(`found ${dependency} in the cache`);
    var dependencies = config.visited[dependency].pathMap.dependencies.map(function (d) {
      return d.resolvedDep;
    });
    tree[dependency] = dependencies;
    config.pathMap.push(config.visited[dependency].pathMap);

    if (config.visited[dependency].missing) {
      config.nonExistent[dependency] = config.visited[dependency].missing;
    }

    if (config.visited[dependency].error) {
      config.errors[dependency] = config.visited[dependency].error;
    }

    dependencies.forEach(function (d) {
      if (!tree[d]) dependenciesStack.push(d);
    });
  }
}
//# sourceMappingURL=index.js.map