"use strict";

function _chai() {
  var data = require("chai");

  _chai = function _chai() {
    return data;
  };

  return data;
}

/**
 * this file had been forked (and changed since then) from https://github.com/dependents/node-detective-es6
 */
var assert = require('assert');

var detective = require('./');

describe('detective-es6', function () {
  var ast = {
    type: 'Program',
    body: [{
      type: 'VariableDeclaration',
      declarations: [{
        type: 'VariableDeclarator',
        id: {
          type: 'Identifier',
          name: 'x'
        },
        init: {
          type: 'Literal',
          value: 4,
          raw: '4'
        }
      }],
      kind: 'let'
    }]
  };
  it('accepts an ast', function () {
    var deps = detective(ast);
    var depsKeys = Object.keys(deps);
    assert(!depsKeys.length);
  });
  it('retrieves the dependencies of es6 modules', function () {
    var deps = detective('import {foo, bar} from "mylib";');
    var depsKeys = Object.keys(deps);
    assert(depsKeys.length === 1);
    assert(depsKeys[0] === 'mylib');
  });
  it('retrieves the re-export dependencies of es6 modules', function () {
    var deps = detective('export {foo, bar} from "mylib";');
    var depsKeys = Object.keys(deps);
    assert(depsKeys.length === 1);
    assert(depsKeys[0] === 'mylib');
  });
  it('retrieves the re-export * dependencies of es6 modules', function () {
    var deps = detective('export * from "mylib";');
    var depsKeys = Object.keys(deps);
    assert(depsKeys.length === 1);
    assert(depsKeys[0] === 'mylib');
  });
  it('handles multiple imports', function () {
    var deps = detective('import {foo, bar} from "mylib";\nimport "mylib2"');
    var depsKeys = Object.keys(deps);
    assert(depsKeys.length === 2);
    assert(depsKeys[0] === 'mylib');
    assert(depsKeys[1] === 'mylib2');
  });
  it('handles default imports', function () {
    var deps = detective('import foo from "foo";');
    var depsKeys = Object.keys(deps);
    assert(depsKeys.length === 1);
    assert(depsKeys[0] === 'foo');
  });
  it('handles dynamic imports', function () {
    var deps = detective('import("foo").then(foo => foo());');
    var depsKeys = Object.keys(deps);
    assert(depsKeys.length === 1);
    assert(depsKeys[0] === 'foo');
  });
  it('should support commonJS syntax', function () {
    var deps = detective('var foo = require("foo");');
    var depsKeys = Object.keys(deps);
    assert(depsKeys.length === 1);
    assert(depsKeys[0] === 'foo');
  });
  it('returns an empty list for empty files', function () {
    var deps = detective('');
    var depsKeys = Object.keys(deps);
    assert.equal(depsKeys.length, 0);
  });
  it('throws when content is not provided', function () {
    assert.throws(function () {
      detective();
    }, Error, 'src not given');
  });
  it('does not throw with jsx in a module', function () {
    assert.doesNotThrow(function () {
      detective("import foo from 'foo'; var templ = <jsx />;");
    });
  });
  it('does not throw on an async ES7 function', function () {
    assert.doesNotThrow(function () {
      detective("import foo from 'foo'; export default async function foo() {}");
    });
  });
  describe('import-specifiers detection (for tree shaking)', function () {
    it('should recognize default imports as default', function () {
      var deps = detective('import foo from "foo";');
      (0, _chai().expect)(deps).to.have.property('foo');
      (0, _chai().expect)(deps.foo).to.have.property('importSpecifiers');
      var importSpecifier = deps.foo.importSpecifiers[0];
      (0, _chai().expect)(importSpecifier.name).to.equal('foo');
      (0, _chai().expect)(importSpecifier.isDefault).to.be.true;
    });
    it('should recognize non-default imports as non-default', function () {
      var deps = detective('import { foo } from "foo";');
      (0, _chai().expect)(deps).to.have.property('foo');
      (0, _chai().expect)(deps.foo).to.have.property('importSpecifiers');
      var importSpecifier = deps.foo.importSpecifiers[0];
      (0, _chai().expect)(importSpecifier.name).to.equal('foo');
      (0, _chai().expect)(importSpecifier.isDefault).to.be.false;
    });
    it('should support export-default-as syntax', function () {
      var deps = detective('export { default as foo } from "foo";');
      (0, _chai().expect)(deps).to.have.property('foo');
      (0, _chai().expect)(deps.foo).to.have.property('importSpecifiers');
      var importSpecifier = deps.foo.importSpecifiers[0];
      (0, _chai().expect)(importSpecifier.name).to.equal('foo');
      (0, _chai().expect)(importSpecifier.isDefault).to.be.true;
    });
    it('should not be supported for CommonJS', function () {
      var deps = detective('const foo = require("foo");');
      (0, _chai().expect)(deps).to.have.property('foo');
      (0, _chai().expect)(deps.foo).to.not.have.property('importSpecifiers');
    });
  });
});
//# sourceMappingURL=index.spec.js.map