"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.processPath = processPath;
exports.default = generateTree;

function _os() {
  var data = _interopRequireDefault(require("os"));

  _os = function _os() {
    return data;
  };

  return data;
}

function _path() {
  var data = _interopRequireDefault(require("path"));

  _path = function _path() {
    return data;
  };

  return data;
}

function _dependencyTree() {
  var data = _interopRequireDefault(require("./dependency-tree"));

  _dependencyTree = function _dependencyTree() {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// most of the functions in this file were taken from the Madge project: https://github.com/pahen/madge
// reasons for not using Madge directly: 1) it has issues with TypeScript on Windows. 2) it has issues with tsx files

/**
 * Check if running on Windows.
 * @type {Boolean}
 */
var isWin = _os().default.platform() === 'win32';
/**
 * Check if path is from NPM folder
 * @param  {String} path
 * @return {Boolean}
 */

function isNpmPathFunc(path) {
  return path.indexOf('node_modules') >= 0;
}
/**
 * Sort tree.
 * @param  {Object} tree
 * @return {Object}
 */


function sort(tree) {
  return Object.keys(tree).sort().reduce(function (acc, id) {
    acc[id] = tree[id].sort();
    return acc;
  }, {});
}
/**
 * Exclude modules from tree using RegExp.
 * @param  {Object} tree
 * @param  {Array} excludeRegExp
 * @return {Object}
 */


function exclude(tree, excludeRegExp) {
  var regExpList = excludeRegExp.map(function (re) {
    return new RegExp(re);
  });

  function regExpFilter(id) {
    return regExpList.findIndex(function (regexp) {
      return regexp.test(id);
    }) < 0;
  }

  return Object.keys(tree).filter(regExpFilter).reduce(function (acc, id) {
    acc[id] = tree[id].filter(regExpFilter);
    return acc;
  }, {});
}
/**
 * Process absolute path and return a shorter one.
 * @param  {String} absPath
 * @param  {Object} cache
 * @param  {String} baseDir
 * @return {String}
 */


function processPath(absPath, cache, baseDir) {
  if (cache[absPath]) {
    return cache[absPath];
  }

  var relPath = _path().default.relative(baseDir, absPath);

  if (isWin) {
    relPath = relPath.replace(/\\/g, '/');
  }

  cache[absPath] = relPath;
  return relPath;
}
/**
 * Convert deep tree produced by dependency-tree to a
 * shallow (one level deep) tree used by madge.
 * @param  {Object} depTree
 * @param  {Object} tree
 * @param  {Object} pathCache
 * @param  {String} baseDir
 * @return {Object}
 */


function convertTreePaths(depTree, pathCache, baseDir) {
  var tree = {};
  Object.keys(depTree).forEach(function (file) {
    tree[processPath(file, pathCache, baseDir)] = depTree[file].map(function (d) {
      return processPath(d, pathCache, baseDir);
    });
  });
  return tree;
}
/**
 * Generate the tree from the given files
 * @param  {Array} files
 * @param config
 * @return {Object}
 */


function generateTree() {
  var files = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var config = arguments.length > 1 ? arguments[1] : undefined;
  var depTree = {};
  var nonExistent = {};
  var npmPaths = {};
  var pathCache = {};
  var pathMap = [];
  var errors = {};
  files.forEach(function (file) {
    if (depTree[file]) {
      return;
    }

    var detective = config.detectiveOptions;

    try {
      var dependencyTreeResult = (0, _dependencyTree().default)({
        filename: file,
        directory: config.baseDir,
        requireConfig: config.requireConfig,
        webpackConfig: config.webpackConfig,
        resolveConfig: config.resolveConfig,
        visited: config.visited,
        errors,
        filter: function filter(dependencyFilePath, traversedFilePath) {
          var dependencyFilterRes = true;
          var isNpmPath = isNpmPathFunc(dependencyFilePath);

          if (config.dependencyFilter) {
            dependencyFilterRes = config.dependencyFilter(dependencyFilePath, traversedFilePath, config.baseDir);
          }

          if (config.includeNpm && isNpmPath) {
            (npmPaths[traversedFilePath] = npmPaths[traversedFilePath] || []).push(dependencyFilePath);
          }

          return !isNpmPath && (dependencyFilterRes || dependencyFilterRes === undefined);
        },
        detective,
        nonExistent,
        pathMap
      });
      Object.assign(depTree, dependencyTreeResult);
    } catch (err) {
      errors[file] = err;
    }
  });
  var tree = convertTreePaths(depTree, pathCache, config.baseDir); // rename errors keys from absolute paths to relative paths

  Object.keys(errors).forEach(function (file) {
    var relativeFile = processPath(file, pathCache, config.baseDir);

    if (relativeFile !== file) {
      errors[relativeFile] = errors[file];
      delete errors[file];
    }
  });
  Object.keys(npmPaths).forEach(function (npmKey) {
    var id = processPath(npmKey, pathCache, config.baseDir); // a file might not be in the tree if it has errors or errors found with its parents

    if (!tree[id]) return;
    npmPaths[npmKey].forEach(function (npmPath) {
      tree[id].push(processPath(npmPath, pathCache, config.baseDir));
    });
  });

  if (config.excludeRegExp) {
    tree = exclude(tree, config.excludeRegExp);
  }

  return {
    madgeTree: sort(tree),
    skipped: nonExistent,
    pathMap,
    errors
  };
}
//# sourceMappingURL=generate-tree-madge.js.map